; *********************************************************************************************
; *********************************************************************************************
;
;	Einfaches Modell eines Datenloggers mit dem ATMega 8
; 	Am ADC0-Eingang liegt das Signal des Temperatursensors, der bereits mit dem Analogcomparator
; 	eingesetzt wurde.
;	Es knnen bis zu 240 Messungen mit einer Zeitbasis von 1 Sekunde bis zu 99 Min und 99 Sekunden
;	eingestellt werden (=6039 Sek. entspr. 14,3 Messungen pro Tag = 16,77 Tage Messdauer)
; 	
;	Autor: Jrgen Grzesina
;	Dateiname: adc.asm
; 	Stand: 11.02.2009
;
; *********************************************************************************************
; *********************************************************************************************
;
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Deklarationen
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

 .equ mega8 = 1						; wir verwenden einen mega8 (wichtig fr die RS232-Routinen)
 .equ debuglevel = 0				; Einstellung verschiedener Testprogramme
	
; Wir verfolgen in diesem Projekt streng das Konzept von Includedateien
; Bei sehr umfangreichen Bibliotheksmodulen gibt es eine Headerdatei, die
; die Declarationen enthlt. In der Regel gengt es, Anpassungen dort vorzunehmen.
; Die eigentlichen Routinen werden am Schluss der Hauptdatei ADC.asm vor den Jobproceduren 
; eingebunden. Die Verfgbarkeit mancher Bibliotheksroutinen ist von der Verfgbarkeit  
; anderer abhngig. Zur Vermeidung von Fehlermeldungen erfolgt die Absicherung ber .if-Konstrukte
;
; Fr die Includes wird kein Listing erstellt
.nolist
.include "m8def.inc"
.include "arithmetik_header.asm"
.include "lcd-routines-bib_header.asm"
.include "rs232inc_header.asm"
; die Datei eeprom_512_inc.asm erfordert keinen Header
; die Datei adc_m8_inc.asm erfordert keinen Header
.list

	.equ led_port = portd
	.equ led_ddr = ddrd
	.equ led1 = 5					; Test LED an PORTD.5

	.def temp1 = r16
	.def temp2 = r17
	.def temp3 = r18
	.def flag = r19
		.equ fblinken = 0
		.equ fgetvalue = 1
;	.def messungen = r20
	.def anzahlwerte = r21
	.def count_low = R26
	.def count_high = R27			; Zeit-Countdown-Zhler fr Intervalle
;
	.equ takt = 16000000
	.equ vwert = 40000-1			; Wert fr ocr1a-Vergleichsregister


.dseg
	jiffies:						; chronischer Registermangel zwingt dazu,
	.byte 1							; manche Variablen in den SRAM-Speicher
	sekunden:						; auszusiedeln
	.byte 1							; Wir definieren hie Speicherstelle und Umfang
	minuten:
	.byte 1
	stunden:
	.byte 1

	zeit_low:
	.byte 1
	zeit_high:
	.byte 1

	maxzahl:
	.byte 1

.eseg
	timebase_low:  .byte 1			; hnlich wie im SRAM werden Speicherstellen im
	timebase_high:  .byte 1			; EEPROM-Bereich reserviert
	number_of_values: .byte 2
	werteliste:
		w_low: .byte 240
		w_high: .byte 240

.cseg
.org 0x0000
; Resetvektor
	rjmp start

.org 0x06; OC1Aaddr					; IRQ-Vektor fr Timer 1
	rjmp Timer1_oc1a				; Sprung zur Serviceroutine


.org 0x001A					; Reset-Einsprung
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Vorbereitungen
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
start:
	ldi temp1, low(ramend)
	out spl,temp1			; stackpointer setzen
	ldi temp1, high(ramend)
	out sph,temp1			; stackpointer setzen

; rs232 initialisieren
	rcall serinit

; ADC initialisieren: Single Conversion, Vorteiler 128 -> 125 kHz Samplefrquenz bei 16MHz
	rcall adc_init			; adc initialisieren
	ldi temp1, 1			; und mit einer ersten Messung beauftragen
	rcall adc_get			; Jetzt werden pro Wandlung nur noch 13 Takte gebraucht

; lcd initialisieren
	rcall lcd_init
	rcall lcd_clear
	rcall lcd_home

; Timer1 fr System-IRQ einrichten
	; Zhler 1 fr die Erzeugung des Zweihundertstel-Sekundentakts vorbereiten
	ldi temp1, high(vwert)			; OCR1A setzen
	out ocr1ah, temp1				; wir lassen Timer1 bis 40000 zhlen
	ldi temp1, low(vwert)			; das schafft er 50 mal pro Sekunde bei 2 MHz Timertakt = Systemtakt/8
	out ocr1al, temp1				; ein Durchgang = 1 Jiffie = 1/50 s

	ldi temp1, (1<<wgm12) | (0<<cs12) | (1<<cs11) | (0<<cs10); CTC ein, Taktquelle = Systemtakt/8 ein, kein Vorteiler
	out tccr1b, temp1
	ldi temp1, 0					; tccr1a alle Bits = 0
	out tccr1a, temp1

	ldi temp1, (1<<ocie1a)			; output compare 1A IRQ zulassen
	out timsk, temp1

	ldi temp1, 0
	sts sekunden, temp1				; Zeitrechnung auf 0 stellen
	sts minuten, temp1
	sts stunden, temp1
	sts jiffies, temp1
	mov flag, temp1					; lsche alle Merker
	mov anzahlwerte,temp1			; Anzahl werte im Register auf Null
									; Werte im EEPROM bleiben unangetastet
	

	sei								; Interrupts zulassen
	
	sbi led_ddr, led1				; Leitung fr LED1 auf Ausgang
	cbi led_port, led1				; LED1 aus
	ldi yl, low(werteliste)
	ldi yh, high(werteliste)		; Adressenverwaltung fr EEPROM	
	ldi temp1, 240					; Maximalanzahl Messwertpaare
	sts maxzahl, temp1				; voreinstellen

; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Hauptschleife
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
loop:
	sbrs flag, fgetvalue	; wurde ein neuer Messwert registriert?
	rjmp main				; nein, dann schauen wir mal nach Tastenbefehlen von der RS232
	rcall werte_sichern		; ja, dann mssen wir erst mal den Wert sichern
Main:
	sbis ucsra, rxc			; wenn das RXC-Bit gesetzt ist, gibt es einen neuen Befehl
	rjmp loop				; Ist nicht der Fall, neuer Schleifendurchlauf
	
	rcall serin_echo		; erstes Zeichen des neuen Befehls einlesen
	cpi temp1, 'e'			; und auf die gltigen Flle berprfen.
	brne job01
job00:
	; setze das Zeitintervall aus dem EEPROM
	rcall timebase_set
	rjmp job99
	; LCDCursor home
job01:
	cpi temp1, 'h'
	brne job02
	rcall lcd_home
	rjmp job99
	; LCD clear
job02:
	cpi temp1, 'c'
	brne job03
	rcall lcd_clear
	rjmp job99
	;LCD initialize
job03:
	cpi temp1, 'i'
	brne job04
	rcall lcd_init
	rjmp job99
	; setze das Zeitintervall in low
job04:
	cpi temp1, 's'
	brne job05
	rcall v24_in_number			; zweistellige Zahl als Bytewert holen
	ldi temp2, low(timebase_low)
	ldi temp3, high(timebase_low)
	rcall eep_write
	rcall v24_number
	rjmp job99
	; setze das Zeitintervall in high
job05:
	cpi temp1, 'm'
	brne job06
	rcall v24_in_number			; zweistellige Zahl als Bytewert holen
	ldi temp2, low(timebase_high)
	ldi temp3, high(timebase_high)
	rcall eep_write
	rcall v24_number
	rjmp job99
	; Messsequenz starten
job06:
	cpi temp1, 'g'
	brne job07
	sbr flag, 1<<fblinken		; lst Blinken der LED durch die ISR von Timer0 aus
	rjmp job99
	; Manueller Abbruch der Messsequenz
job07:
	cpi temp1, 'b'
	brne job08
	cbr flag, 1<<fblinken		; stoppt Blinken der LED
	rjmp job99
	; Anzeige der Anzahl Messwerte im EEPROM und im Register anzahlwerte
job08:
	cpi temp1, 'a'
	brne job09
	ldi temp2, high(number_of_values)
	ldi temp1, low(number_of_values)	; hherwertigere Einheit lesen
	rcall eep_read
	rcall v24_number
	ldi temp1, ' '
	rcall serout
	mov temp1, anzahlwerte
	rcall v24_number
	rjmp job99
	; sende die Anzahl Werte, die Zeitbasis und die Werte selbst ber die RS232
job09:
	cpi temp1, 't'
	brne job10
	rcall werte_transfer
	rjmp job99
	; Setze die Maximalzahl automatisch aufgenommener Messwerte
job10:
	cpi temp1, 'o'
	brne job11
	rcall maxzahl_setzen
	rjmp job99
	; Lsche die Messwerte im EEPROM (eigentlich nur die Anzahl Werte)
job11:
	cpi temp1, 'l'
	brne job12
	rcall werte_loeschen
	rjmp job99
job12:
job99:
	ldi temp1, 0x0a
	rcall serout
	ldi temp1, 0x0d		; neue Zeile
	rcall serout			
	; jobs aus den ISR-Proceduren
	rjmp loop

; ********************************************************
; ****************** IRQ - Jobroutinen *******************
; ********************************************************
; Zeitsteuerung
;
Timer1_oc1a:
	push temp1						; temp1 sichern
	in temp1, sreg					; Statusregister sichern
	push temp1
	push temp2

	lds temp1, jiffies				; Jiffies laden
	inc temp1						; Jiffies erhhen
	cpi temp1, 50					; mit 50 vergleichen
	breq weiter_mit_sekunden		; 50 erreicht, dann mit Sekunden weiter
	sts jiffies, temp1				; noch keine 50, -> speichern und
	rjmp end_isr					; fertig
									
weiter_mit_sekunden:				; es ist der wert 50 erreicht
	clr temp1						; Jiffies lschen 
	sts jiffies, temp1				; abspeichern

; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; 	LED blinken lassen, wenn Flag fblinken gesetzt ist
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	sbrs flag, fblinken				; Messung luft, wenn fblinken = 1
	rjmp loeschen					; Messung luft nicht, LED aus!
led_blinken:
	sbic led_port, led1	
	rjmp loeschen					; LED1 blinken lassen
	sbi led_port, led1
	rjmp bit_fertig
loeschen:
	cbi led_port, led1

; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;	Messen ist aktiv, wenn LED blinkt
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

bit_fertig:
	sbrs flag, fblinken				; Messung luft, wenn fblinken = 1
	rjmp getvalue_done				; Messung luft nicht, also keine Auswertung!
	; count_high:low -1
	sbiw R26, 1 					; Countdownzhler fr Zeitintervall verringern
	; Count_high:low = 0?
	breq getvalue					; auf Null gezhlt? dann wird's Zeit fr eine Messung
	rjmp getvalue_done				; nicht Null, dann im Moment keine Messung ntig
getvalue:
	sbr flag, 1<<fgetvalue			; Erst mal Flag setzen, dass neue Messung erfolgen muss
									; ja, dann Wert holen  (das macht die Jobschleife!)
	inc anzahlwerte					; wir haben einen Messwert mehr
									; wertezhler kleiner als Maximum?
	lds temp2, maxzahl
	cp anzahlwerte, temp2
	brlo set_countdown				;
									; nein, fblinken aus, messen beendet
	cbr flag, 1<<fblinken
	rjmp getvalue_done
set_countdown:
									; ja, countdown neu setzen und weiter
	lds count_low, zeit_low			; Der Wert steht als Low/Highbyte im SRAM
	lds count_high, zeit_high

getvalue_done:						; Messwertbehandlung beendet

; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;	Ganz normal weiter in der Zeitrechnung
; ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	lds temp1, sekunden
	inc temp1					; sekunden erhhen

	cpi temp1, 60				; 60 Sekunden voll?
	breq weiter_mit_minuten		; dann Minutenbertrag
	sts sekunden, temp1			; nein, noch nicht
	rjmp fertig					; dann fertig

weiter_mit_minuten:
	clr temp1					; Sekunden lschen und
	sts sekunden, temp1			; die Null merken

	lds temp1, minuten			; wie bei Sekunden
	inc temp1	
	cpi temp1, 60		
	breq weiter_mit_stunden
	sts minuten, temp1
	rjmp fertig

weiter_mit_stunden:
	clr temp1
	sts minuten, temp1

	lds temp1, stunden			; wie bei Sekunden
	inc temp1
	cpi temp1, 24
	breq tag_fertig
	sts stunden, temp1
	rjmp fertig
	
tag_fertig:
	clr temp1
	sts stunden, temp1			; Zeitrechnung startet von vorne

fertig:

end_isr:
	pop temp2
	pop temp1
	out sreg, temp1					; status register restaurieren
	pop temp1						; temp1 zurck
	reti							; irq-routine beenden


.nolist
.include "arithmetik.asm"
.include "adc_m8_inc.asm"
.include "lcd-routines-bib.asm"
.include "rs232inc.asm"
.include "eeprom_512_inc.asm"
.list


; ********************************************************
; ********************************************************
; Job-Proceduren
; ********************************************************
; ********************************************************
; Zeitintervall setzen
timebase_set:
	ldi temp2, high(timebase_high)
	ldi temp1, low(timebase_high)	; hherwertigere Einheit lesen
	rcall eep_read
	clr R23
	mov R22, temp1
	clr R25
	ldi R24, 60
	rcall mulx16					; hherwertigere Einheit mal 60
	ldi temp2, high(timebase_low)
	ldi temp1, low(timebase_low)	; niederwertigere Einheit lesen
	rcall eep_read
	clr temp2
	add R2, temp1
	adc R3, temp2					; zum Produktwert addieren
	sts zeit_low, R2
	sts zeit_high, R3				; Zeitintervall abspeichern und
	mov count_low, r2
	mov count_high, r3				; in countdown-zhler schreiben
	mov r4,r3						; Werte in AkkuB justieren
	mov r3,r2
	rcall v24_binout				; auf rs232 ausgeben
	ret


; Aktuellen Messwert ins EEPROM, Zeiger und Anzahlwerte nachfhren
werte_sichern: 						; y-Pointer zeigt auf nchste freie EEPROM-Adresse
	ldi temp1, 3					; hole den Mittelwert von 2 hoch 3 = 8
	rcall adc_get					; Einzelmessungen vom ADC
	mov temp1, r3					;
	mov temp2, yl
	mov temp3, yh
	rcall eep_write					; Lowbyte im EEPROM ablegen 
	adiw R28, 1

	mov temp1, r4
	mov temp2, yl
	mov temp3, yh
	rcall eep_write					; Highbyte im EEPROM ablegen 
	adiw R28, 1
	
	mov temp1, anzahlwerte			; wurde in der ISR zu Timer1 bereits erhht
	ldi temp2, low(number_of_values)
	ldi temp3, high(number_of_values)
	rcall eep_write					; Anzahl der Werte merken

	cbr flag, 1<<fgetvalue			; Flag fr neuen Messwert rcksetzen

	rcall lcd_clear					; Anzeige lschen, Cursor home
	mov temp1, anzahlwerte
	rcall lcd_number				; Messertnummer ausgeben und
	ldi temp1, 0
	ldi temp2, 1					; in Spalte0, Zeile2 den Messwert
	rcall lcd_position
	rcall lcd_binout
	ret

; Anzahlwerte, Zeitdelay und Werte an die RS232
werte_transfer: 					
	ldi temp2, high(number_of_values)
	ldi temp1, low(number_of_values)	; hherwertigere Einheit lesen
	rcall eep_read
	mov anzahlwerte, temp1			; als Zhler merken
	rcall v24_number				; auf rs232 ausgeben
	rcall v24_lfcr					; CR ausgeben
	tst anzahlwerte					; Anzahlwerte = 0?
	breq wt02						; ja, -> fertig
	ldi temp2, high(timebase_high)
	ldi temp1, low(timebase_high)	; hherwertigere Zeitangabe lesen
	rcall eep_read
	clr R23			
	mov R22, temp1					; nach Faktor 1 kopieren
	clr R25
	ldi R24, 60						; Faktor 2 = 60
	rcall mulx16					; Hoher Zeitwert mal 60
	ldi temp2, high(timebase_low)
	ldi temp1, low(timebase_low)	; niederwertigere Zeitangabe lesen
	rcall eep_read
	clr temp2						; 0 nach temp2
	add R2, temp1					; niedriger Zeitwert + LSB vom Produkt
	adc R3, temp2					; MSB vom Produkt und evtl. bertrag (Carry) bercksichtigen
	mov r4,r3						; Anzahl von Sekunden (Minuten)
	mov r3,r2						; LSB und MSB in AkkuB richtig justieren
	rcall v24_binout				; auf rs232 ausgeben
	rcall v24_lfcr					; CR ausgeben

	ldi yl, low(werteliste)			; y-Pointer setzen auf
	ldi yh, high(werteliste)		; AdressenBasis fr Werteliste im EEPROM	
	clr R5							; HSB in AkkuB vorsorglich lschen (nur 2 Byte ntig)
wt01:
	mov temp1, yl
	mov temp2, yh					; Adresse im EEPROM
	rcall eep_read					; Wert Lowbyte lesen
	mov R3, temp1					; nach AkkuB Low kopieren fr Ausgaberoutine
	adiw y, 1						; Zeiger ins EEPROM erhhen
	mov temp1, yl
	mov temp2, yh					; Adresse im EEPROM
	rcall eep_read					; Highbyte lesen
	mov R4, temp1					; nach AkkuB middle kopieren
	adiw y, 1						; Zeiger ins EEPROM fr nchsten Durchgang erhhen
	rcall v24_binout				; AkkuB auf rs232 ausgeben
	rcall v24_lfcr					; Trennzeichen LF+CR
	dec anzahlwerte					; Anzahl herunterzhlen
	brne wt01						; wiederhole solange anzahlwerte noch grer 0
wt02:
	ret


maxzahl_setzen:
	ldi zl, low(2*str_prompt01)
	ldi zh, high(2*str_prompt01)	; Zeiger auf Eingabe-Prompt
	rcall v24_flash_string			; String ausgeben
	rcall v24_read_int				; Zahl einlesen, Wert steht in AkkuB (R5:R3)
	rcall v24_lfcr					; Zeilenvorschub und Wagenrcklauf
	and R4, r4						; Ist R4 = 0?
	brne ms01						; nein, dann Fehler, weil Eingabe >= 256
	mov temp1, R3
	cpi temp1, 241					; Ist R3 (LSB) > 240?
	brsh ms01						; ja, dann Fehler, weil Eingabe zu gro
	tst r3							; ist R3 = 0?
	brne ms00a						; nein, dann OK
	ldi temp1, 1					; Null geht nicht
	mov r3, temp1					; wir setzen R3 = 1
ms00a:
	sts maxzahl, R3					; Merke die Obergrenze fr Messwerte im SRAM
	rcall v24_lfcr					; neue Zeile
	rcall v24_number				; registrierten Wert zurckmelden
	ldi zl, low(2*str_registriert)	; mit 
	ldi zh, high(2*str_registriert)	; Besttigungstext
ms00:
	rcall v24_flash_string
	ret
ms01:
	ldi zl, low(2*str_fehler)		; y-Zeiger auf Adresse der Fehlermeldung setzen
	ldi zh, high(2*str_fehler)
	rjmp ms00


werte_loeschen:						; Es gengt, die Anzahl Werte auf 0 zu setzen
	ldi temp1, 0
	mov anzahlwerte, temp1					;
	ldi temp2, low(number_of_values)
	ldi temp3, high(number_of_values)
	rcall eep_write					; auch im EEPROM
	ret

str_registriert:
	.db " wurde eingestellt.", 0

str_fehler:
	.db 0x0a, 0x0d, " F E H L E R !!! Wert groesser 240 !!", 0

str_prompt01:
	.db 0x08, "Bitte eine Zahl groesser 0 aber kleiner 240 eingeben: ",0
; ********************************************************
; ********************************************************







.exit




